# Generated by Django 4.2.8 on 2024-01-23 08:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="SkosCollection",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Collection label or name",
                        max_length=300,
                        verbose_name="skos:prefLabel",
                    ),
                ),
                (
                    "label_lang",
                    models.CharField(
                        blank=True,
                        default="en",
                        help_text="Language of preferred label given above",
                        max_length=3,
                        verbose_name="skos:prefLabel language",
                    ),
                ),
                (
                    "creator",
                    models.TextField(
                        blank=True,
                        help_text="Person or organisation that created this collectionIf more than one list all using a semicolon ;",
                        verbose_name="dc:creator",
                    ),
                ),
                (
                    "contributor",
                    models.TextField(
                        blank=True,
                        help_text="Person or organisation that made contributions to the collectionIf more than one list all using a semicolon ;",
                        verbose_name="dc:contributor",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="collections.skoscollection",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SkosCollectionContentObject",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("object_id", models.PositiveIntegerField()),
                (
                    "collection",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="collections.skoscollection",
                    ),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
    ]
