{% extends "apis_core/apis_entities/abstractentity_detail.html" %}
{% load static %}
{% load generic %}

{% block scripts %}
  {{ block.super }}
  <script src="{% static "js/E53_Place_map.js" %}"></script>
{% endblock %}

{% block col-zero %}
  <div class="card">
    {% template_list object "_card_table.html" as template_list %}
    <div class="card-body">{% include template_list %}</div>
    <div class="card-body pt-0">
      <table class="table table-hover table-bordered">
        {% if object.latitude and object.longitude %}
          <td id="map" class="height-300">
            <ul hidden>
              <li data-longitude="{{ object.longitude|floatformat:"4u" }}"
                  data-latitude="{{ object.latitude|floatformat:"4u" }}">{{ object }}</li>
            </ul>
          </td>
        {% endif %}
      </table>
    </div>
    <div class="card-footer">{% include "apis_entities/partials/linked_open_data.html" %}</div>
  </div>
{% endblock %}
