{% extends basetemplate|default:"base.html" %}
{% load crispy_forms_tags %}
{% load i18n %}

{% block content %}
  <div class="container">
    <div class="card">
      <div class="card-header">
        {% blocktranslate with url=object.get_absolute_url %}Select to merge into <a href="{{ url }}">{{ object }}</a>{% endblocktranslate %}
      </div>
      <div class="card-body">{% crispy form form.helper %}</div>
    </div>
  </div>
{% endblock content %}
