{% extends "generic/generic_content.html" %}
{% load static %}

{% block styles %}
  {{ block.super }}
  <link href="{% static 'css/apis_entities.css' %}" rel="stylesheet" />
{% endblock styles %}

{% block content %}
  <div class="container-fluid">
    <div class="card">
      <div class="card-header">

        {% block card-header-content %}
          <div id="single-object-header" class="row">
            <div class="col-md-2">{% include "apis_entities/partials/prev_url.html" %}</div>
            <div class="col-md-8">
              <h2>
                {% include "apis_entities/partials/listview_url.html" %}
                {{ object }}

                {% block object-actions %}
                {% endblock object-actions %}

              </h2>
            </div>
            <div class="col-md-2">{% include "apis_entities/partials/next_url.html" %}</div>
          </div>
        {% endblock card-header-content %}

        {% block navigation %}
          {% include "apis_entities/partials/entity_base_nav.html" %}
          {% include "apis_entities/partials/entity_actions_nav.html" %}
        {% endblock navigation %}

      </div>
      <div class="card-body">

        {% block card-body %}
          <div class="row">
            <div class="col-md-4">

              {% block col-zero %}
              {% endblock col-zero %}

            </div>
            <div class="col-md-8">

              {% block col-one %}
              {% endblock col-one %}

            </div>
          </div>
        {% endblock card-body %}

      </div>
    </div>
  </div>
{% endblock content %}
