<!DOCTYPE html>
{% load static %}
{% load i18n %}
<html lang="en">
  <link href="{% static 'css/maintenance.css' %}" rel="stylesheet" />
  <head>
    <meta charset="UTF-8">
    <title>{% translate "Site Maintenance" %}</title>
  </head>
  <body>
    <article>
      <h1>{% translate "We&rsquo;ll be back soon!" %}</h1>
      <div>
        <p>
          {% translate "Sorry for the inconvenience but we&rsquo;re performing some maintenance at the moment. We&rsquo;ll be back online shortly!" %}
        </p>
        <p>&mdash; {% translate "The Team" %}</p>
      </div>
    </article>
  </body>
</html>
